;/******************************/;
;/* Example For : JRHC08JL3 V2 */;
;/* Controller  : 68HC908JL3   */;
;/* Assembler   : C32D4CL.EXE  */;
;/* Demo Test (I2C)RTC-DS1307  */;
;/* LCD Interface With 74HC595 */;
;/******************************/;
;
;/*************************/;
;/* Register HC08 Equates */;
;/*************************/;
;
PORTA       EQU     $0000                   ; Port-A Data Register
PORTB       EQU     $0001                   ; Port-B Data Register
PORTD       EQU     $0003                   ; Port-D Data Regsiter
DDRA        EQU     $0004                   ; Port-A Direction Register
DDRB        EQU     $0005                   ; Port-B Direction Register
DDRD        EQU     $0007                   ; Port-D Direction Register
PDCR        EQU     $000A                   ; Port-D Control Register
PTAPUE      EQU     $000D                   ; Port-A Pull-Up Enable
KBSCR       EQU     $001A                   ; Keyboard Status Control
KBIER       EQU     $001B                   ; Keyboard Interrupt Enable
INTSCR      EQU     $001D                   ; IRQ Status Control
CONFIG2     EQU     $001E                   ; Config Register2
CONFIG1     EQU     $001F                   ; Config Register1
TSC         EQU     $0020                   ; TIM Status & Control
TCNTH       EQU     $0021                   ; TIM Counter High
TCNTL       EQU     $0022                   ; TIM Counter Low
TMODH       EQU     $0023                   ; TIM Counter Modulo High
TMODL       EQU     $0024                   ; TIM Counter Modulo Low
TSC0        EQU     $0025                   ; TIM CH0 Status & Control
TCH0H       EQU     $0026                   ; TIM CH0 High
TCH0L       EQU     $0027                   ; TIM CH0 Low
TSC1        EQU     $0028                   ; TIM CH1 Status & Control
TCH1H       EQU     $0029                   ; TIM CH1 High
TCH1L       EQU     $002A                   ; TIM CH1 Low
ADSCR       EQU     $003C                   ; ADC Status & Control
ADR         EQU     $003D                   ; ADC Data Read
ADICLK      EQU     $003E                   ; ADC Input Clock
BSR         EQU     $FE00                   ; Break Status Register
RSR         EQU     $FE01                   ; Reset Status Register
BFCR        EQU     $FE03                   ; Break Flag Control
INT1        EQU     $FE04                   ; Interrupt Status1
INT2        EQU     $FE05                   ; Interrupt Status2
INT3        EQU     $FE06                   ; Interrupt Status3
FLCR        EQU     $FE08                   ; Flash Control Register
FLBPR       EQU     $FE09                   ; Flash Block Protect
BRKH        EQU     $FE0C                   ; Break Address High
BRKL        EQU     $FE0D                   ; Break Address Low
BRKSCR      EQU     $FE0E                   ; Break Status & Control
COPCTL      EQU     $FFFF                   ; Reset COP
;/* End Equate I/O Register of HC08 */;
;
;/********************/;
;/* Register Equates */;
;/********************/;
;
PORT_LCD    EQU     PORTD                   ; PD0,PD1,PD2 Control 74HC595
LATCH_LCD   EQU     00H                     ; Bit0 of Port Control LCD
SHIFT_LCD   EQU     01H                     ; Bit1 of Port Control LCD
DATA_LCD    EQU     02H                     ; Bit2 of Port Control LCD
RS_LCD      EQU     02H                     ; Bit2 of 74HC595
EN_LCD      EQU     03H                     ; Bit3 of 74HC595
D4_LCD      EQU     04H                     ; Bit4 of 74HC595
D5_LCD      EQU     05H                     ; Bit5 of 74HC595
D6_LCD      EQU     06H                     ; Bit6 of 74HC595
D7_LCD      EQU     07H                     ; Bit7 of 74HC595
;
PORT_I2C    EQU     PORTD                   ; PD6,PD7 Control I2C Device
PDIR_I2C    EQU     DDRD                    ; PORTD Direction
I2C_SCL     EQU     06H                     ; PD6 = SCL
I2C_SDA     EQU     07H                     ; PD7 = SDA
SECOND      EQU     00H                     ; Second 00-59
MINUTE      EQU     01H                     ; Minute 00-59
HOUR        EQU     02H                     ; Hour 01-12 or 00-23
DAY         EQU     03H                     ; Day 01-07
DATE        EQU     04H                     ; Data 01-28/29 or 01-30 or 01-31
MONTH       EQU     05H                     ; Month 01-12
YEAR        EQU     06H                     ; Year 00-99
CTRL_RTC    EQU     07H                     ; Sware Wave Control Output
;
PORT_DAC    EQU     PORTD                   ; PD3,PD4,PD5 Control DAC
ENA_DAC     EQU     03H                     ; Bit3 "0" = Enable CS# For DAC
DIN_DAC     EQU     04H                     ; Bit4 = Data Input For DAC
SCK_DAC     EQU     05H                     ; Bit5 = Serial Clock For DAC
;
PORT_KEY    EQU     PORTA                   ; PA0..3 = Read Row Matrix Key
PORT_SEL    EQU     PORTA                   ; Select CS0..3 For DAC / Col0..3 For Key
SEL_139A    EQU     04H                     ; Bit4 = A-Select 74HCT139
SEL_139B    EQU     05H                     ; Bit5 = B-Select 74HCT139

							
            ORG     80H
LCD_SIGN    DFS     1                       ; LCD Data Signal Buffer
DLY_BUFF    DFS     1                       ; Delay Buffer

            ;/****************************/;
            ;/* 68HC908JL3 Port I/O Test */;
            ;/****************************/;
            ;
            CPU     "68HC08.TBL"            ; 68HC908JL3 Table
            HOF     "MOT8"                  ; S19 Hex Format
            ORG     $EC00                   ; Start Main (68HC908JL3)
            ;
RESET:      BSET    0,CONFIG1               ; Disable COP Watch-Dog
            MOV     #0,LCD_SIGN             ; Clear Signal Buffer
            LDA     #11111111B              ; Port PD7..0 = Output
            STA     DDRD
            LDA     #00110000B              ; Port PA5..4 = Output,PA3..0 = Input
            STA     DDRA
            LDA     #00001111B              ; PA3..0 Input Pull-Up Enable
            STA     PTAPUE
            ;
            BCLR    SCK_DAC,PORT_DAC        ; Initial DAC Control
            BCLR    DIN_DAC,PORT_DAC
            BSET    ENA_DAC,PORT_DAC        ; Disable CS# Output
            BCLR    SEL_139A,PORT_SEL       ; Select-A = "0"
            BCLR    SEL_139B,PORT_SEL       ; Select-B = "0"
            ;
            BCLR    DATA_LCD,PORT_LCD       ; Initial 74HC595
            BCLR    LATCH_LCD,PORT_LCD
            BCLR    SHIFT_LCD,PORT_LCD
            ;
            BCLR    I2C_SCL,PORT_I2C        ; Initial Bus For Stop I2C Device
            BCLR    I2C_SDA,PORT_I2C
            BSET    I2C_SCL,PORT_I2C
            BSET    I2C_SDA,PORT_I2C        ; Stop Condition (SCL=1,SDA=1)
            ;
            ;/**************************/;
            ;/* Power-on Delay For LCD */;
            ;/**************************/;
            LDHX    #0000H                  ; Power-On Delay Count
PWR_DLY0:   AIX     #1                      ; H:X + 1
            CPHX    #0FFFFH
            BNE     PWR_DLY0
            JSR     INIT_LCD                ; Initial LCD
            ;
            LDA     #$00                    ; Cursor Line1 (16x2)
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            DFB     "JR-68HC908JL3 V2",00H
            LDA     #$40                    ; Cursor Line2 (16x2)
            JSR     GOTO_LCD                
            JSR     PRINT_LCD
            DFB     "TIME :  HR:MN:SC",00H
            ;
            LDX     #SECOND                 ; Check Clock Halt Flag
            LDA     #00000000B
            JSR     RTC_WRITE               ; Enable Clock
            ;
LOOP_RTC:   LDA     #$48                    ; Display Hour
            JSR     GOTO_LCD
            ;
            LDX     #HOUR                   ; Get Hour Time
            JSR     RTC_READ1
            PSHA
            NSA
            AND     #03H
            ADD     #30H
            JSR     WR_LCD
            PULA
            AND     #0FH
            ADD     #30H
            JSR     WR_LCD
            ;
            LDA     #$4B                    ; Display Minute
            JSR     GOTO_LCD
            ;
            LDX     #MINUTE                 ; Get Minute Time
            JSR     RTC_READ1
            PSHA
            NSA
            AND     #07H
            ADD     #30H
            JSR     WR_LCD
            PULA
            AND     #0FH
            ADD     #30H
            JSR     WR_LCD
            ;
            LDA     #$4E                    ; Display Second
            JSR     GOTO_LCD
            ;
            LDX     #SECOND                 ; Get Second Time
            JSR     RTC_READ1
            PSHA
            NSA
            AND     #07H
            ADD     #30H
            JSR     WR_LCD
            PULA
            AND     #0FH
            ADD     #30H
            JSR     WR_LCD
            ;
            MOV     #250,DLY_BUFF           ; Delay 1-Second
            JSR     DLY_MSEC
            MOV     #250,DLY_BUFF
            JSR     DLY_MSEC
            MOV     #250,DLY_BUFF
            JSR     DLY_MSEC
            MOV     #250,DLY_BUFF
            JSR     DLY_MSEC
            JMP     LOOP_RTC

;/********************/;
;/* Initial Char LCD */;
;/* 4-Bit Interface  */;
;/* RS    = Bit-2    */;
;/* EN    = Bit-3    */;
;/* D4..7 = Bit-4..7 */;
;/********************/;
;
INIT_LCD:   LDA     #33H                    ; DL = 1 ,DL =1
            JSR     WR_INS
            LDA     #32H                    ; DL = 1 ,DL =0
            JSR     WR_INS
            LDA     #28H                    ; Function set
            JSR     WR_INS                  ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDA     #0CH                    ; Display on/off Control
            JSR     WR_INS                  ; Entry display,cursor off,not blink
            LDA     #06H                    ; Entry mode set
            JSR     WR_INS                  ; I/D=1 Increment,S=0 cursor shift
CLR_LCD:    LDA     #01H                    ; Clear Display
            JSR     WR_INS                  ; Clear Display,Set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin EN CLCD */;
;/* Pin EN LCD = Bit-3 */;
;/* Active Chip select */;
;/**********************/;
;
ENA_LCD:    JSR     BUSY                    ; Busy Delay
            BSET    EN_LCD,LCD_SIGN         ; Enable Chips Select
            JSR     SHF_SIGN
            NOP
            NOP
            BCLR    EN_LCD,LCD_SIGN         ; Disable Chips Select
            JSR     SHF_SIGN
            RTS

;/*******************/;
;/* Busy CLCD Delay */;
;/* Defualt = 100uS */;
;/* Oscilator 32MHz */;
;/* 1Cycle = 1/8MHz */;
;/* 100uS = 800Cycle*/;
;/*******************/;
;
BUSY:       PSHX                            ; 2 Cycle
            LDX     #198                    ; 2 Cycle
BUSY1:      NOP                             ; 1 Cycle
            DBNZX   BUSY1                   ; 3 Cycle (198x4 =792)
            PULX                            ; 2 Cycle
            RTS                             ; 4 Cycle

;/**********************/;
;/* Write ASCII to LCD */;
;/* Input : ACC(ASCII) */;
;/**********************/;
;
WR_LCD:     PSHA                            ; Save Data Low Byte
            BCLR    D4_LCD,LCD_SIGN         ; Clear old Data
            BCLR    D5_LCD,LCD_SIGN
            BCLR    D6_LCD,LCD_SIGN
            BCLR    D7_LCD,LCD_SIGN
            AND     #$F0
            ORA     LCD_SIGN                ; High Byte
            STA     LCD_SIGN
            BSET    RS_LCD,LCD_SIGN         ; Set RS For Write Data Select
            JSR     SHF_SIGN
            JSR     ENA_LCD
            PULA                            ; Restore Data Low Byte
            ;
            NSA                             ; Nibble Swap A
            BCLR    D4_LCD,LCD_SIGN         ; Clear old Data
            BCLR    D5_LCD,LCD_SIGN
            BCLR    D6_LCD,LCD_SIGN
            BCLR    D7_LCD,LCD_SIGN
            AND     #$F0
            ORA     LCD_SIGN                ; Low Byte
            STA     LCD_SIGN
            BSET    RS_LCD,LCD_SIGN         ; Set RS For Write Data Select
            JSR     SHF_SIGN
            JSR     ENA_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS:     PSHA                            ; Save Data Low Byte
            BCLR    D4_LCD,LCD_SIGN         ; Clear old Data
            BCLR    D5_LCD,LCD_SIGN
            BCLR    D6_LCD,LCD_SIGN
            BCLR    D7_LCD,LCD_SIGN
            AND     #$F0
            ORA     LCD_SIGN                ; High Byte
            STA     LCD_SIGN
            BCLR    RS_LCD,LCD_SIGN         ; Clear RS For Write Instruction
            JSR     SHF_SIGN
            JSR     ENA_LCD
            PULA                            ; Restore Data Low Byte
            ;
            NSA                             ; Nibble Swap A
            BCLR    D4_LCD,LCD_SIGN         ; Clear old Data
            BCLR    D5_LCD,LCD_SIGN
            BCLR    D6_LCD,LCD_SIGN
            BCLR    D7_LCD,LCD_SIGN
            AND     #$F0
            ORA     LCD_SIGN                ; Low Byte
            STA     LCD_SIGN
            BCLR    RS_LCD,LCD_SIGN         ; Clear RS For Write Instruction
            JSR     SHF_SIGN
            JSR     ENA_LCD
            MOV     #2,DLY_BUFF             ; Wait Command Ready = 2mS
            JSR     DLY_MSEC
            RTS
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD:   ORA     #$80                    ; Set ACC.7
            JSR     WR_INS
            RTS

;/********************/;
;/* Shift LCD Signal */;
;/* Input : LCD_SIGN */;
;/********************/;
;
SHF_SIGN:   PSHA                            ; Save Rigister
            PSHX
            TPA                             ; CCR -> ACC
            PSHA
            ;
            LDX     #8                      ; 8 Bit Counter Shift
            LDA     LCD_SIGN                ; Get LCD Signal Buffer
SHF_DAT0:   ROLA                            ; Shift D7..D0 (D7 First)
            BCS     SHF_DAT1                ; Jump if Data = "1"
            BCLR    DATA_LCD,PORT_LCD
            BRA     SHF_DAT2
            ;
SHF_DAT1:   BSET    DATA_LCD,PORT_LCD
SHF_DAT2:   NOP	    		
            BSET    SHIFT_LCD,PORT_LCD      ; Shift Data 74HC595
            NOP
            NOP
            BCLR    SHIFT_LCD,PORT_LCD
            DBNZX   SHF_DAT0
            ;
            BSET    LATCH_LCD,PORT_LCD      ; Latch Data 74HC595
            NOP
            NOP
            BCLR    LATCH_LCD,PORT_LCD
            ;
            PULA                            ; Restore Register
            TAP                             ; A -> CCR
            PULX
            PULA
            RTS

;/*******************/;
;/* Delay Time 1 mS */;
;/* CPU Bus = 8 MHz */;
;/* 1Cycle = 1/8MHz */;
;/* 1mS = 8000Cycle */;
;/* Input :DLY_BUFF */;
;/*******************/;
;
DLY_MSEC:   PSHH                            ; Save Register (2 Cycle)
            PSHX                            ; 2 Cycle
            PSHA                            ; 2 Cycle
            TPA                             ; CCR -> A (1 Cycle)
            PSHA                            ; 2 Cycle
            LDA     DLY_BUFF                ; 3 Cycle (Get Delay)
            ;
DLY_MS0:    LDHX    #886                    ; 3 Cycle
DLY_MS1:    NOP                             ; 1 Cycle
            AIX     #-1                     ; H:X - 1 (2 Cycle)
            CPHX    #0                      ; 3 Cycle
            BNE     DLY_MS1                 ; 3 Cycle ( 9 x 886 = 8000 Cycle)
            DBNZA   DLY_MS0                 ; 3 Cycle
            ;
            PULA                            ; 2 Cycle
            TAP                             ; A -> CCR (2 Cycle)
            PULA                            ; 2 Cycle
            PULX                            ; 2 Cycle
            PULH                            ; 2 Cycle
            RTS                             ; 4 Cycle

;/************************/;
;/* Convert HEX to ASCII */;
;/* Input  : A (HEX)     */;
;/* Output : A (ASCII)   */;
;/************************/;
;
HEX_ASCII:  AND     #0FH                ; Clear High Nibble
            CMP     #9
            BHI     HEX_ASC1            ; A..F Jump
            ADD     #30H                ; Convert 0..9 to '0..9'
            BRA     HEX_ASC2
            ;
HEX_ASC1:   SUB     #9                  ; Convert A..F to 1..6
            ADD     #40H                ; Convert 1..6 to 'A..F'
            ;
HEX_ASC2:   RTS

;/********************************/;
;/* Print ASCII Data to LCD Port */;
;/* Usage    : JSR  PRINT_LCD    */;
;/*          : DFB  'xxxxxxxx'   */;
;/*          : DFB  $00          */;
;/* Note     : Last Byte = $00   */;
;/********************************/;
;
PRINT_LCD:  PULH                        ; Pop Stack PCH
            PULX                        ; Pop Stack PCL
            PSHA                        ; Save ACC
            ;
PRN_LCD1:   LDA     0,X                 ; Get Data For Print
            BEQ     PRN_LCD2            ; If Last Byte Jump
            JSR     WR_LCD              ; Write Data to LCD
            INCX
            BRA     PRN_LCD1
            ;
PRN_LCD2:   INCX                        ; Skip Last Byte(00)
            ;
            PULA                        ; Restore ACC
            PSHX                        ; Push PCL to Stack
            PSHH                        ; Push PCH to Stack
            RTS

;/****************************/;
;/* I2C-BUS DS1307 Interface */;
;/* Write Data to I2C-RTC    */;
;/* Input : X (Address)      */;
;/*       : A (Data)         */;
;/****************************/;
;
RTC_WRITE:  PSHA                            ; Save Data to Write
            PSHX                            ; Save RTC Address
            TPA                             ; CCR -> A
            PSHA                            ; Save CCR
            ;
            BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            BSET    I2C_SCL,PDIR_I2C        ; SCL = Output Direction
            ;
RTC_WR0:    BSET    I2C_SCL,PORT_I2C        ; Initial Bus For Start
            BSET    I2C_SDA,PORT_I2C
            BCLR    I2C_SDA,PORT_I2C        ; Start Condition THD:STA(600nS)
            BCLR    I2C_SCL,PORT_I2C
            ;
            ;/* Ready to Start 1st Byte Send */;
            LDX     #8                      ; Write Control Byte 8-Bit Count
            LDA     #11010000B              ; 1101000,Write
            ;
RTC_WR1:    BCLR    I2C_SCL,PORT_I2C        ;/* Change Data to SDA Signal */;
            JSR     I2C_WRLF                ; SDA Shift-Out D7..D0 (D7 First)
            DBNZX   RTC_WR1
            ;
            BCLR    I2C_SCL,PORT_I2C        ;*/ Falling 9'th Clock For ACK */;
            BCLR    I2C_SDA,PDIR_I2C        ; SDA = Input (Read ACK)
            BSET    I2C_SCL,PORT_I2C        ;*/ Rising 9'th Clock for Get ACK */;
            BRCLR   I2C_SDA,PORT_I2C,RTC_WR2
            BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            BRA     RTC_WR0                 ; Resend Control Byte Until ACK
            ;
            ;/* Ready to Start 2nd Byte Send */;
RTC_WR2:    BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            LDX     #8                      ; Write Address Count
            LDA     2,SP                    ; Get RTC Address
            ;
RTC_WR3:    BCLR    I2C_SCL,PORT_I2C        ;/* Change Data to SDA Signal */;
            JSR     I2C_WRLF                ; SDA Shift-Out D7..D0 (D7 First)
            DBNZX   RTC_WR3
            ;
            BCLR    I2C_SCL,PORT_I2C        ;*/ Falling 9'th Clock For ACK */;
            BCLR    I2C_SDA,PDIR_I2C        ; SDA = Input (Read ACK)
            BSET    I2C_SCL,PORT_I2C        ;*/ Rising 9'th Clock for Get ACK */;
            BRCLR   I2C_SDA,PORT_I2C,RTC_WR4
            BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            BRA     RTC_WR6                 ; Exit
            ;
RTC_WR4:    BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            LDX     #8                      ; Write Data 8-Bit Count
            LDA     3,SP                    ; Get Data For Write
            ;
RTC_WR5:    BCLR    I2C_SCL,PORT_I2C        ;/* Change Data to SDA Signal */;
            JSR     I2C_WRLF                ; SDA Shift-Out D7..D0 (D7 First)
            DBNZX   RTC_WR5
            ;
            BCLR    I2C_SCL,PORT_I2C        ;*/ Falling 9'th Clock For ACK */;
            BSET    I2C_SDA,PORT_I2C        ; Release Data Line
            BSET    I2C_SCL,PORT_I2C        ;*/ Rising 9'th Clock for Get ACK */;
            ;
RTC_WR6:    BCLR    I2C_SCL,PORT_I2C        ; Falling Stop Clock
            BCLR    I2C_SDA,PORT_I2C        ; Wait For Stop
            BSET    I2C_SCL,PORT_I2C        ; Stop Condition (SCL=1,SDA=1)
            BSET    I2C_SDA,PORT_I2C
            ;
            PULA                            ; Restore CCR
            TAP                             ; A -> CCR
            PULX                            ; Restore RTC Address
            PULA                            ; Restore RTC Data
            RTS

;/*****************************/;
;/* I2C-BUS  DS1307 Interface */;
;/* Read Data From I2C-RTC    */;
;/* -RTC_READ1 = Random Read  */;
;/*  Input  : X (Address)     */;
;/*  Output : A (Data)        */;
;/* -RTC_READ2 = Current Read */;
;/*  Output : ACC (Data)      */;
;/*****************************/;
;
RTC_READ1:  AIS     #-1                     ; Reserve Stack 1 Byte
            PSHX                            ; Save RTC Address
            TPA                             ; CCR -> A
            PSHA                            ; Save CCR
            ;
            BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            BSET    I2C_SCL,PDIR_I2C        ; SCL = Output Direction
            ;
RTC_RD0:    BSET    I2C_SCL,PORT_I2C        ; Initial Bus For Start
            BSET    I2C_SDA,PORT_I2C
            BCLR    I2C_SDA,PORT_I2C        ; Start Condition THD:STA(600nS)
            BCLR    I2C_SCL,PORT_I2C
            ;
            ;/* Ready to Start 1st Byte Send */;
            LDX     #8                      ; Write Control Byte 8-Bit Count
            LDA     #11010000B              ; 1101000,Write
            ;
RTC_RD1:    BCLR    I2C_SCL,PORT_I2C        ;/* Change Data to SDA Signal */;
            JSR     I2C_WRLF                ; SDA Shift-Out D7..D0 (D7 First)
            DBNZX   RTC_RD1                 ; Loop to Write Control Byte
            ;
            BCLR    I2C_SCL,PORT_I2C        ;*/ Falling 9'th Clock For ACK */;
            BCLR    I2C_SDA,PDIR_I2C        ; SDA = Input (Read ACK)
            BSET    I2C_SCL,PORT_I2C        ;*/ Rising 9'th Clock for Get ACK */;
            BRCLR   I2C_SDA,PORT_I2C,RTC_RD2
            BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            BRA     RTC_RD0                 ; Resend Control Byte Until ACK
            ;
RTC_RD2:    BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            LDX     #8                      ; Write Address Count
            LDA     2,SP                    ; Get RTC Address
            ;
RTC_RD3:    BCLR    I2C_SCL,PORT_I2C        ;/* Change Data to SDA Signal */;
            JSR     I2C_WRLF                ; SDA Shift-Out D7..D0 (D7 First)
            DBNZX   RTC_RD3                 ; Loop to Write Addr High-Byte
            ;
            BCLR    I2C_SCL,PORT_I2C        ;*/ Falling 9'th Clock For ACK */;
            BCLR    I2C_SDA,PDIR_I2C        ; SDA = Input (Read ACK)
            BSET    I2C_SCL,PORT_I2C        ;*/ Rising 9'th Clock for Get ACK */;
            BRCLR   I2C_SDA,PORT_I2C,RTC_RD4
            BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            BRA     RTC_RD10                ; Exit
            ;
            ;/* New Start For Send Control Byte Read Command */;
RTC_RD4:    BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            BCLR    I2C_SCL,PORT_I2C        ; Falling New Start Clock
            BSET    I2C_SCL,PORT_I2C        ; Initial Bus For Start
            BSET    I2C_SDA,PORT_I2C
            BCLR    I2C_SDA,PORT_I2C        ; Start Condition THD:STA(600nS)
            BCLR    I2C_SCL,PORT_I2C
            ;
            LDX     #8                      ; Write Control Byte 8-Bit Count
            LDA     #11010001B              ; 1101000,Read
            BRA     RTC_RD7
            ;
            ;/* Current Address Read Operation */;
RTC_READ2:  AIS     #-1                     ; Reserve Stack 1 Byte
            PSHX                            ; Save RTC Address
            TPA                             ; CCR -> A
            PSHA                            ; Save CCR
            BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            BSET    I2C_SCL,PDIR_I2C        ; SCL = Output Direction
            ;
            BSET    I2C_SCL,PORT_I2C        ; Initial Bus For Start
            BSET    I2C_SDA,PORT_I2C
            BCLR    I2C_SDA,PORT_I2C        ; Start Condition THD:STA(600nS)
            BCLR    I2C_SCL,PORT_I2C
            ;
            LDX     #8                      ; Write Control Byte 8-Bit Count
            LDA     #11010001B              ; 1101-Page000,Read
            ;
RTC_RD7:    BCLR    I2C_SCL,PORT_I2C        ;/* Change Data to SDA Signal */;
            JSR     I2C_WRLF                ; SDA Shift-Out D7..D0 (D7 First)
            DBNZX   RTC_RD7                 ; Loop to Write Control Byte
            ;
            BCLR    I2C_SCL,PORT_I2C        ;*/ Falling 9'th Clock For ACK */;
            BCLR    I2C_SDA,PDIR_I2C        ; SDA = Input (Read ACK)
            BSET    I2C_SCL,PORT_I2C        ;*/ Rising 9'th Clock for Get ACK */;
            BRCLR   I2C_SDA,PORT_I2C,RTC_RD8
            BSET    I2C_SDA,PDIR_I2C        ; SDA = Output Direction
            BRA     RTC_RD10                ; Exit
            ;
RTC_RD8:    LDX     #8                      ; Read Data 8-Bit Count
            CLRA                            ; Clera Buffer For Read
            ;
RTC_RD9:    BCLR    I2C_SCL,PORT_I2C        ;/* Wait to Receive SDA Signal */;
            JSR     I2C_RDLF                ; SDA Shift-IN D7..D0 (D0 First)
            DBNZX   RTC_RD9                 ; Loop to Read Data 8-Bit
            STA     3,SP                    ; Save Result
            ;
            BCLR    I2C_SCL,PORT_I2C        ;*/ Falling 9'th Clock For ACK */;
            BSET    I2C_SDA,PORT_I2C        ; Not ACK At Stop Read
            BSET    I2C_SCL,PORT_I2C        ;*/ Rising 9'th Clock for Send ACK */;
            ;
RTC_RD10:   BCLR    I2C_SCL,PORT_I2C        ; Falling Stop Clock
            BCLR    I2C_SDA,PORT_I2C        ; Wait For Stop
            BSET    I2C_SCL,PORT_I2C        ; Stop Condition (SCL=1,SDA=1)
            BSET    I2C_SDA,PORT_I2C
            ;
            PULA                            ; Restore CCR
            TAP                             ; A -> CCR
            PULX                            ; Restore RTC Address
            PULA                            ; Get Result to ACC
            RTS

;/**************************/;
;/* Write SDA & Shift Left */;
;/* Input : ACC            */;
;/**************************/;
;
I2C_WRLF:   ROLA                            ; Shift D7..D0 (D7 First)
            BCS     I2C_WRL0                ; Jump if Data = "1"
            BCLR    I2C_SDA,PORT_I2C        ; SDA = "0"
            BRA     I2C_WRL1
I2C_WRL0:   BSET    I2C_SDA,PORT_I2C        ; SDA = "1"
I2C_WRL1:   NOP
            NOP
            BSET    I2C_SCL,PORT_I2C        ; Shift Data to I2C Device
            NOP
            NOP
            RTS

;/*************************/;
;/* Read SDA & Shift Left */;
;/* Output : ACC          */;
;/*************************/;
;
I2C_RDLF:   BCLR    I2C_SDA,PDIR_I2C        ; SDA = Input (Read)
            NOP
            NOP
            BSET    I2C_SCL,PORT_I2C        ; Read Data From I2C-Device
            NOP
            NOP
            BRSET   I2C_SDA,PORT_I2C,I2C_RDL0
            CLC                             ; C = "0"
            BRA     I2C_RDL1
I2C_RDL0:   SEC                             ; C = "1"
I2C_RDL1:   ROLA                            ; Shift D7..D0 (D7 First)
            BSET    I2C_SCL,PORT_I2C        ; Shift Data to I2C Device
            NOP
            NOP
            RTS

            ;/***************/;
            ;/* User Vector */;
            ;/***************/;
            ;
            ORG     $FFF6               ; Security Code
            DFB     "JR08JLV2"
            ORG     $FFFE               ; Reset Vector
            DWM     RESET

            END

